<?php
/*
	filter.inc
	part of m0n0wall (http://m0n0.ch/wall)
	
	Copyright (C) 2003 Manuel Kasper <mk@neon1.net>.
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
	
	1. Redistributions of source code must retain the above copyright notice,
	   this list of conditions and the following disclaimer.
	
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
	AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/
	
/* include all configuration functions */
require_once("functions.inc");
	
function filter_resync() {
	global $config, $g;
	
	mwexec("/sbin/ipf -y");
}

function filter_ipmon_start() {
	global $config, $g;
	
	mwexec("/sbin/ipmon -sD");
}

function filter_configure() {
	global $config, $g;
	
	if ($g['booting'])
		echo "Configuring firewall... ";
	
	/* generate ipnat rules */
	$ipnatrules = filter_nat_rules_generate();
	
	/* load ipnat rules */
	$fd = popen("/sbin/ipnat -C -f - > /dev/null 2>&1", "w");
	if (!$fd) {
		printf("Cannot open /sbin/ipnat in filter_configure()\n");
		return 1;
	}
		
	fwrite($fd, $ipnatrules);
	pclose($fd);
	
	/* generate ipf rules */
	$ipfrules = filter_rules_generate();
	
	/* load ipnat rules */
	$fd = popen("/sbin/ipf -Fa -l block -f - > /dev/null 2>&1", "w");
	if (!$fd) {
		printf("Cannot open /sbin/ipf in filter_configure()\n");
		return 1;
	}
		
	fwrite($fd, $ipfrules);
	pclose($fd);
	
	if ($g['booting'])
		echo "done\n";
	
	return 0;
}

function filter_flush_nat_table() {
	global $config, $g;
	
	return mwexec("/sbin/ipnat -F");
}

function filter_flush_state_table() {
	global $config, $g;
	
	return mwexec("/sbin/ipf -FS");
}

function filter_nat_rules_generate_if($if, $sa, $sn, $mssclamp) {
	$natrule = <<<EOD
map $if $sa/$sn -> 0/32 proxy port ftp ftp/tcp $mssclamp
map $if $sa/$sn -> 0/32 portmap tcp/udp auto $mssclamp
map $if $sa/$sn -> 0/32 $mssclamp

EOD;

	return $natrule;
}

function filter_nat_rules_generate() {
	global $config, $g;
	
	$wancfg = $config['interfaces']['wan'];
	$lancfg = $config['interfaces']['lan'];
	
	$pptpdcfg = $config['pptpd'];
	$wanif = get_real_wan_interface();
	
	if ($wancfg['ipaddr'] == "pppoe")
		$mssclamp = "mssclamp 1452";
	else
		$mssclamp = "";
	
	$lansa = gen_subnet($lancfg['ipaddr'], $lancfg['subnet']);
	
	$natrules = "";
	
	/* any 1:1 mappings? */
	if (is_array($config['nat']['onetoone'])) {
		foreach ($config['nat']['onetoone'] as $natent) {
			$natrules .= "bimap {$wanif} {$natent['internal']}/32 -> {$natent['external']}/32\n";
		}
	}
	
	/* outbound rules - advanced or standard */
	if (isset($config['nat']['advancedoutbound']['enable'])) {
		/* advanced outbound rules */
		if (is_array($config['nat']['advancedoutbound']['rule'])) {
			foreach ($config['nat']['advancedoutbound']['rule'] as $obent) {
				list($sa,$sn) = explode("/", $obent['source']);
				$natrules .= filter_nat_rules_generate_if($wanif, $sa, $sn, $mssclamp);
			}
		}	
	} else {
		/* standard outbound rules (one for each interface) */
		$natrules .= filter_nat_rules_generate_if($wanif, $lansa, $lancfg['subnet'], $mssclamp);
	
		/* optional interfaces */
		for ($i = 1; isset($config['interfaces']['opt' . $i]); $i++) {
			$optcfg = $config['interfaces']['opt' . $i];
			
			if (isset($optcfg['enable'])) {
				$optsa = gen_subnet($optcfg['ipaddr'], $optcfg['subnet']);
				$natrules .= filter_nat_rules_generate_if($wanif, $optsa, $optcfg['subnet'], $mssclamp);
			}
		}
	}
	
	$natrules .= "\n# User-defined rules follow\n\n";
	
	/* DIAG: add ipv6 NAT, if requested */
	if (isset($config['diag']['ipv6nat']['enable'])) {
		$natrules .= "rdr $wanif 0/0 port 0 -> " . 
		"{$config['diag']['ipv6nat']['ipaddr']} port 0 ipv6\n";
	}
	
	if (isset($config['nat']['rule']))
		foreach ($config['nat']['rule'] as $rule) {
		
		$extport = explode("-", $rule['external-port']);
		
		if ((!$extport[1]) || ($extport[0] == $extport[1])) {
			$natrules .=
				"rdr $wanif 0/0 port {$extport[0]} -> {$rule['target']} " .
				"port {$rule['local-port']} {$rule['protocol']}";
		} else {
			$natrules .=
				"rdr $wanif 0/0 port {$extport[0]}-{$extport[1]} " .
				"-> {$rule['target']} " .
				"port {$rule['local-port']} {$rule['protocol']}";
		}
		
		$natrules .= " {$mssclamp}";
			
		$natrules .= "\n";
	}
	
	if ($pptpdcfg['mode']) {
	
		if ($pptpdcfg['mode'] == "server")
			$pptpdtarget = "127.0.0.1";
		else
			$pptpdtarget = $pptpdcfg['redir'];
	
		$natrules .= <<<EOD

# PPTP
rdr $wanif 0/0 port 0 -> $pptpdtarget port 0 gre
rdr $wanif 0/0 port 1723 -> $pptpdtarget port 1723 tcp {$mssclamp}

EOD;
	}
	
	return $natrules;
}

function filter_rules_generate() {
	global $config, $g;
	
	$wancfg = $config['interfaces']['wan'];
	$lancfg = $config['interfaces']['lan'];
	$pptpdcfg = $config['pptpd'];
	
	$lanif = $lancfg['if'];
	$wanif = get_real_wan_interface();
	
	/* rule groups (optional interfaces: see below) */
	$ifgroups = array("lan" => 100, "wan" => 200);
	
	$lanip = $lancfg['ipaddr'];
	$lansa = gen_subnet($lancfg['ipaddr'], $lancfg['subnet']);
	$lansn = $lancfg['subnet'];
	
	/* optional interfaces */
	$optcfg = array();
	
	for ($i = 1; isset($config['interfaces']['opt' . $i]); $i++) {
		$oc = $config['interfaces']['opt' . $i];
		
		if (isset($oc['enable']) && $oc['if'] && (!$oc['bridge'])) {
			$oic = array();
			$oic['ip'] = $oc['ipaddr'];
			$oic['if'] = $oc['if'];
			$oic['sa'] = gen_subnet($oc['ipaddr'], $oc['subnet']);
			$oic['sn'] = $oc['subnet'];
			
			$optcfg['opt' . $i] = $oic;
			
			$ifgroups['opt' . $i] = ($i * 100) + 200;
		}
	}
	
	if ($pptpdcfg['mode'] == "server") {
		$pptpip = $pptpdcfg['localip'];
		$pptpsa = $pptpdcfg['remoteip'];
		$pptpsn = $g['pptp_subnet'];
	}
	
	$ipfrules = <<<EOD
# loopback
pass in quick on lo0 all
pass out quick on lo0 all

# block short packets
block in quick all with short

# block IP options
block in quick all with ipopts

# allow access to DHCP server on LAN
pass in quick on $lanif proto udp from any port = 68 to 255.255.255.255 port = 67
pass in quick on $lanif proto udp from any port = 68 to $lanip port = 67
pass out quick on $lanif proto udp from $lanip port = 67 to any port = 68

# WAN spoof check
block in log quick on $wanif from $lansa/$lansn to any

EOD;

	foreach ($optcfg as $oc) {
		$ipfrules .= "block in log quick on $wanif from {$oc['sa']}/{$oc['sn']} to any\n";
	}
	
	/* allow PPTP traffic if PPTP client is enabled on WAN */
	if ($wancfg['ipaddr'] == "pptp") {
		$ipfrules .= <<<EOD

# allow PPTP client
pass in quick on {$wancfg['if']} proto gre from any to any
pass out quick on {$wancfg['if']} proto gre from any to any
pass in quick on {$wancfg['if']} proto tcp from any port = 1723 to any
pass out quick on {$wancfg['if']} proto tcp from any to any port = 1723

EOD;
	}

	$ipfrules .= <<<EOD

# allow our DHCP client out to the WAN
# XXX - should be more restrictive
# (not possible at the moment - need 'me' like in ipfw)
pass out quick on $wanif proto udp from any port = 68 to any port = 67
block in log quick on $wanif proto udp from any port = 67 to $lansa/$lansn port = 68
pass in quick on $wanif proto udp from any port = 67 to any port = 68

# LAN/OPT spoof check (needs to be after DHCP because of broadcast addresses)

EOD;

	/* LAN spoof check */	
	$ipfrules .= filter_rules_spoofcheck_generate('lan', $lanif, $lansa, $lansn);

	/* OPT spoof check */
	foreach ($optcfg as $on => $oc) {
		$ipfrules .= filter_rules_spoofcheck_generate($on, $oc['if'], $oc['sa'], $oc['sn']);
	}

	if (($pptpdcfg['mode'] == "server") && isset($pptpdcfg['radius']['enable'])) {
		$ipfrules .= <<<EOD

# allow outgoing RADIUS queries from firewall
pass out quick proto udp from any to any port 1811 >< 1814 keep state

EOD;
	}

	/* XXX - the first section is only needed because ipf refuses to
		parse rules that have "flags S/SAFR" and proto "tcp/udp" set because
		UDP does not have flags, but we still want to offer the TCP/UDP protocol
		option to the user */
		
	$ipfrules .= <<<EOD

# Block TCP packets that do not mark the start of a connection
skip 1 in proto tcp all flags S/SAFR
block in quick proto tcp all

#---------------------------------------------------------------------------
# group head 100/150 - LAN interface
#---------------------------------------------------------------------------
block in quick on $lanif all head 100
block out quick on $lanif all head 150

#---------------------------------------------------------------------------
# group head 200/250 - WAN interface
#---------------------------------------------------------------------------
block in log quick on $wanif all head 200
block out quick on $wanif all head 250

EOD;

	/* group heads for optional interfaces */
	foreach ($optcfg as $on => $oc) {
	
		$ingroup = $ifgroups[$on];
		$outgroup = $ifgroups[$on] + 50;
	
		$ipfrules .= <<<EOD
		
#---------------------------------------------------------------------------
# group head {$ingroup}/{$outgroup} - {$on} interface
#---------------------------------------------------------------------------
block in quick on {$oc['if']} all head {$ingroup}
block out quick on {$oc['if']} all head {$outgroup}

# allow the firewall access to DNS service on servers in {$on}
pass out quick proto udp from {$oc['ip']} to {$oc['sa']}/{$oc['sn']} port = 53 keep state group {$outgroup}

EOD;
	
	}
	
	$ipfrules .= <<<EOD

# make sure the user cannot lock himselves out of the webGUI
pass in quick from $lansa/$lansn to $lanip keep state group 100

EOD;
	
	/* IPsec enabled? */
	if (isset($config['ipsec']['enable']) && 
		is_array($config['ipsec']['tunnel']) &&
		count($config['ipsec']['tunnel'])) {
		
		/* add rules for each tunnel */
		foreach ($config['ipsec']['tunnel'] as $tunnel) {
		
		if ($tunnel['p2']['protocol'] == 'esp') {
		
		$ipfrules .= <<<EOD

# Pass ESP packets in/out
pass in quick proto esp from {$tunnel['remote-gateway']} to {$wancfg['ipaddr']} group 200
pass out quick proto esp from {$wancfg['ipaddr']} to {$tunnel['remote-gateway']} group 250

EOD;
		} else if ($tunnel['p2']['protocol'] == 'ah') {
		
		$ipfrules .= <<<EOD

# Pass AH packets in/out
pass in quick proto ah from {$tunnel['remote-gateway']} to {$wancfg['ipaddr']} group 200
pass out quick proto ah from {$wancfg['ipaddr']} to {$tunnel['remote-gateway']} group 250

EOD;
		}
		
		$ipfrules .= <<<EOD

# Pass ipencap packets in/out
pass in quick proto ipencap from {$tunnel['remote-gateway']} to {$wancfg['ipaddr']} group 200
pass out quick proto ipencap from {$wancfg['ipaddr']} to {$tunnel['remote-gateway']} group 250

# Pass IKE packets
pass in quick proto udp from {$tunnel['remote-gateway']} to {$wancfg['ipaddr']} port = 500 group 200
pass out quick proto udp from {$wancfg['ipaddr']} port = 500 to {$tunnel['remote-gateway']} group 250

# Pass packets from tunnel
pass in quick from {$tunnel['remote-subnet']} to {$lansa}/{$lansn} keep state group 200
	
EOD;
		}
	}
	
	/* block private networks on WAN? */
	if (isset($config['interfaces']['wan']['blockpriv'])) {
		$ipfrules .= <<<EOD

# block anything from private networks on WAN interface
block in quick from 10.0.0.0/8 to any group 200
block in quick from 127.0.0.0/8 to any group 200
block in quick from 172.16.0.0/12 to any group 200
block in quick from 192.168.0.0/16 to any group 200

EOD;
	}
	
	/* PPTPd enabled? */
	if ($pptpdcfg['mode']) {
	
		if ($pptpdcfg['mode'] == "server")
			$pptpdtarget = "127.0.0.1";
		else
			$pptpdtarget = $pptpdcfg['redir'];
			
		$ipfrules .= <<<EOD

# PPTP rules
pass in quick proto gre from any to $pptpdtarget keep state group 200
pass in quick proto tcp from any to $pptpdtarget port = 1723 keep state group 200

EOD;
	}
	
	$i = 0;
	
	$ipfrules .= "\n# User-defined rules follow\n";
	
	if (isset($config['filter']['rule']))
		foreach ($config['filter']['rule'] as $rule) {
		
		/* does the rule deal with a PPTP interface? */
		if ($rule['interface'] == "pptp") {
		
			if ($pptpdcfg['mode'] != "server")
				continue;
				
			$nif = $g['n_pptp_units'];
			$ispptp = true;
		} else {
			
			if (strstr($rule['interface'], "opt")) {
				if (!array_key_exists($rule['interface'], $optcfg))
					continue;
			}
			
			$nif = 1;
			$ispptp = false;
		}
		
		if ($pptpdcfg['mode'] != "server") {
			if (($rule['source']['network'] == "pptp") ||
				($rule['destination']['network'] == "pptp"))
					continue;
		}
		
		if (strstr($rule['source']['network'], "opt")) {
			if (!array_key_exists($rule['source']['network'], $optcfg))
				continue;
		}
		if (strstr($rule['destination']['network'], "opt")) {
			if (!array_key_exists($rule['destination']['network'], $optcfg))
				continue;
		}
		
		for ($iif = 0; $iif < $nif; $iif++) {
			
			if (!$ispptp) {
			
				$groupnum = $ifgroups[$rule['interface']];
				
				if (!$groupnum) {
					printf("Invalid interface name in rule $i\n");
					break;
				}
			}
			
			$line = "pass in quick ";
			
			if ($ispptp) {
				$line .= "on ng" . ($iif+1) . " ";
			}
			
			if (isset($rule['protocol'])) {
				$line .= "proto {$rule['protocol']} ";
			}
			
			/* source address */
			if (isset($rule['source']['any'])) {
				$src = "any";
			} else if ($rule['source']['network']) {
				
				if (strstr($rule['source']['network'], "opt")) {
					$src = $optcfg[$rule['source']['network']]['sa'] . "/" .
						$optcfg[$rule['source']['network']]['sn'];
				} else {
					switch ($rule['source']['network']) {
						case 'lan':
							$src = "$lansa/$lansn";
							break;
						case 'pptp':
							$src = "$pptpsa/$pptpsn";
							break;
					}
				}
			} else if ($rule['source']['address']) {
				$src = $rule['source']['address'];
			}
			
			if (!$src) {
				printf("No source address found in rule $i\n");
				break;
			}
			
			if (isset($rule['source']['not'])) {
				$line .= "from !$src ";
			} else {
				$line .= "from $src ";
			}
			
			if (in_array($rule['protocol'], array("tcp","udp","tcp/udp"))) {
				
				if ($rule['source']['port']) {
					$srcport = explode("-", $rule['source']['port']);
					
					if ((!$srcport[1]) || ($srcport[0] == $srcport[1])) {
						$line .= "port = {$srcport[0]} ";
					} else {
						$srcport[0]--;
						$srcport[1]++;
						$line .= "port {$srcport[0]} >< {$srcport[1]} ";
					}
				}
			}
			
			/* destination address */
			if (isset($rule['destination']['any'])) {
				$dst = "any";
			} else if ($rule['destination']['network']) {
				
				if (strstr($rule['destination']['network'], "opt")) {
					$dst = $optcfg[$rule['destination']['network']]['sa'] . "/" .
						$optcfg[$rule['destination']['network']]['sn'];
				} else {
					switch ($rule['destination']['network']) {
						case 'lan':
							$dst = "$lansa/$lansn";
							break;
						case 'pptp':
							$dst = "$pptpsa/$pptpsn";
							break;
					}
				}
			} else if ($rule['destination']['address']) {
				$dst = $rule['destination']['address'];
			}
			
			if (!$dst) {
				printf("No destination address found in rule $i\n");
				break;
			}
			
			if (isset($rule['destination']['not'])) {
				$line .= "to !$dst ";
			} else {
				$line .= "to $dst ";
			}
			
			if (in_array($rule['protocol'], array("tcp","udp","tcp/udp"))) {
				
				if ($rule['destination']['port']) {
					$dstport = explode("-", $rule['destination']['port']);
					
					if ((!$dstport[1]) || ($dstport[0] == $dstport[1])) {
						$line .= "port = {$dstport[0]} ";
					} else {
						$dstport[0]--;
						$dstport[1]++;
						$line .= "port {$dstport[0]} >< {$dstport[1]} ";
					}
				}
			}
			
			$line .= "keep state ";
			
			if (isset($rule['frags']))
				$line .= "keep frags ";
				
			if (!$ispptp) {
				$line .= "group $groupnum ";
			}
			
			$line .= "\n";
			
			$ipfrules .= $line;
		}
		
		$i++;
	}
	
	$ipfrules .= <<<EOD

# allow the firewall access to DNS service on servers in LAN
pass out quick proto udp from $lanip to $lansa/$lansn port = 53 keep state group 150

# allow the firewall to ping LAN hosts (for DHCP server)
pass out quick proto icmp from $lanip to $lansa/$lansn keep state group 150

# allow all outgoing (from firewall) on WAN
pass out quick proto tcp from any to any keep state group 250
pass out quick proto udp from any to any keep state group 250
pass out quick proto icmp from any to any keep state group 250

EOD;

	/* allow the firewall access to DNS service on servers on optional interfaces */
	$ipfrules .= <<<EOD

# allow the firewall access to DNS service on servers on OPT
EOD;

	foreach ($optcfg as $on => $oc) {
		$groupnum = $ifgroups[$on] + 50;
		$ipfrules .= "pass out quick proto udp from {$oc['ip']} to " . 
			"{$oc['sa']}/{$oc['sn']} port = 53 keep state group {$groupnum}\n";
	}
	
	$ipfrules .= <<<EOD
	
#---------------------------------------------------------------------------
# default rules (just to be sure)
#---------------------------------------------------------------------------
block in quick all
block out quick all

EOD;
	
	return $ipfrules;
}

function filter_rules_spoofcheck_generate($ifname, $if, $sa, $sn) {

	global $g, $config;
	
	$ipfrules = "";

	if (is_array($config['staticroutes']['route']) && count($config['staticroutes']['route'])) {
		/* count rules */
		$n = 1;
		foreach ($config['staticroutes']['route'] as $route) {
			if ($route['interface'] == $ifname)
				$n++;
		}
		
		/* output skip rules */
		foreach ($config['staticroutes']['route'] as $route) {
			if ($route['interface'] == $ifname) {
				$ipfrules .= "skip $n in on $if from {$route['network']} to any\n";
				$n--;
			}
		}
		$ipfrules .= "skip 1 in on $if from $sa/$sn to any\n";
		$ipfrules .= "block in quick on $if all\n";
	} else {
		$ipfrules .= "block in quick on $if from ! $sa/$sn to any\n";
	}
	
	return $ipfrules;
}

?>
