<?php
/*
	shaper.inc
	part of m0n0wall (http://m0n0.ch/wall)
	
	Copyright (C) 2003 Manuel Kasper <mk@neon1.net>.
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
	
	1. Redistributions of source code must retain the above copyright notice,
	   this list of conditions and the following disclaimer.
	
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
	AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/
	
/* include all configuration functions */
require_once("functions.inc");
	
function shaper_configure() {
	global $config, $g;
	
	if (isset($config['shaper']['enable'])) {
	
		if ($g['booting'])
			echo "Starting traffic shaper... ";
		
		/* generate shaper rules */
		$shaperrules = shaper_rules_generate();
		
		/* make sure ipfw and dummynet are loaded */
		mwexec("/sbin/kldload ipfw");
		mwexec("/sbin/kldload dummynet");
		
		/* change one_pass to 0 so ipfw doesn't stop checking after
		   a pipe rule has matched */
		mwexec("/sbin/sysctl net.inet.ip.fw.one_pass=0");
		
		/* load shaper rules */
		mwexec("/sbin/ipfw -f flush");
		
		/* XXX - seems like ipfw cannot accept rules directly on stdin,
		   so we have to write them to a temporary file first */
		$fd = fopen("{$g['tmp_path']}/ipfw.rules", "w");
		if (!$fd) {
			printf("Cannot open ipfw.rules in shaper_configure()\n");
			return 1;
		}
			
		fwrite($fd, $shaperrules);
		fclose($fd);
		
		mwexec("/sbin/ipfw {$g['tmp_path']}/ipfw.rules");
		
		unlink("{$g['tmp_path']}/ipfw.rules");
	
		if ($g['booting'])
			echo "done\n";
		
	} else {
		/* unload ipfw and dummynet */
		mwexec("/sbin/kldunload dummynet");
		mwexec("/sbin/kldunload ipfw");
	}
	
	return 0;
}

function shaper_rules_generate() {
	global $config, $g;

	$wancfg = $config['interfaces']['wan'];
	$lancfg = $config['interfaces']['lan'];
	$pptpdcfg = $config['pptpd'];
	
	$lanif = $lancfg['if'];
	$wanif = get_real_wan_interface();
	
	$lanip = $lancfg['ipaddr'];
	$lansa = gen_subnet($lancfg['ipaddr'], $lancfg['subnet']);
	$lansn = $lancfg['subnet'];
	
	/* optional interfaces */
	$optcfg = array();
	
	for ($i = 1; isset($config['interfaces']['opt' . $i]); $i++) {
		$oc = $config['interfaces']['opt' . $i];
		
		if (isset($oc['enable']) && $oc['if'] && (!$oc['bridge'])) {
			$oic = array();
			$oic['ip'] = $oc['ipaddr'];
			$oic['if'] = $oc['if'];
			$oic['sa'] = gen_subnet($oc['ipaddr'], $oc['subnet']);
			$oic['sn'] = $oc['subnet'];
			
			$optcfg['opt' . $i] = $oic;
		}
	}
	
	if ($pptpdcfg['mode'] == "server") {
		$pptpip = $pptpdcfg['localip'];
		$pptpsa = $pptpdcfg['remoteip'];
		$pptpsn = $g['pptp_subnet'];
	}
	
	$pipei = 1;
	
	/* add a rule to pass all traffic from/to the firewall,
	   so the user cannot lock himselves out of the webGUI */
	$shaperrules = "add pass all from $lanip to any\n";
	$shaperrules .= "add pass all from any to $lanip\n";
	
	if (isset($config['shaper']['rule']))	
		foreach ($config['shaper']['rule'] as $rule) {
		
		/* does the rule deal with a PPTP interface? */
		if ($rule['interface'] == "pptp") {
		
			if ($pptpdcfg['mode'] != "server")
				continue;
				
			$nif = $g['n_pptp_units'];
			$ispptp = true;
		} else {
			
			if (strstr($rule['interface'], "opt")) {
				if (!array_key_exists($rule['interface'], $optcfg))
					continue;
			}
		
			$nif = 1;
			$ispptp = false;
		}
		
		if ($pptpdcfg['mode'] != "server") {
			if (($rule['source']['network'] == "pptp") ||
				($rule['destination']['network'] == "pptp"))
					continue;
		}
		
		if (strstr($rule['source']['network'], "opt")) {
			if (!array_key_exists($rule['source']['network'], $optcfg))
				continue;
		}
		if (strstr($rule['destination']['network'], "opt")) {
			if (!array_key_exists($rule['destination']['network'], $optcfg))
				continue;
		}
		
		for ($iif = 0; $iif < $nif; $iif++) {
			
			$line = "add pipe $pipei ";
			
			if (isset($rule['protocol'])) {
				$line .= "{$rule['protocol']} ";
			} else {
				$line .= "all ";
			}
			
			/* source address */
			if (isset($rule['source']['any'])) {
				$src = "any";
			} else if ($rule['source']['network']) {
				
				if (strstr($rule['source']['network'], "opt")) {
					$src = $optcfg[$rule['source']['network']]['sa'] . "/" .
						$optcfg[$rule['source']['network']]['sn'];
				} else {
					switch ($rule['source']['network']) {
						case 'lan':
							$src = "$lansa/$lansn";
							break;
						case 'pptp':
							$src = "$pptpsa/$pptpsn";
							break;
					}
				}
			} else if ($rule['source']['address']) {
				$src = $rule['source']['address'];
			}
			
			if (!$src) {
				printf("No source address found in rule $i\n");
				break;
			}
			
			if (isset($rule['source']['not'])) {
				$line .= "from not $src ";
			} else {
				$line .= "from $src ";
			}
			
			if (in_array($rule['protocol'], array("tcp","udp"))) {
				
				if ($rule['source']['port']) {
					$srcport = explode("-", $rule['source']['port']);
					
					if ((!$srcport[1]) || ($srcport[0] == $srcport[1])) {
						$line .= "{$srcport[0]} ";
					} else {
						$line .= "{$srcport[0]}-{$srcport[1]} ";
					}
				}
			}
			
			/* destination address */
			if (isset($rule['destination']['any'])) {
				$dst = "any";
			} else if ($rule['destination']['network']) {
				
				if (strstr($rule['destination']['network'], "opt")) {
					$dst = $optcfg[$rule['destination']['network']]['sa'] . "/" .
						$optcfg[$rule['destination']['network']]['sn'];
				} else {
					switch ($rule['destination']['network']) {
						case 'lan':
							$dst = "$lansa/$lansn";
							break;
						case 'pptp':
							$dst = "$pptpsa/$pptpsn";
							break;
					}
				}
			} else if ($rule['destination']['address']) {
				$dst = $rule['destination']['address'];
			}
			
			if (!$dst) {
				printf("No destination address found in rule $i\n");
				break;
			}
			
			if (isset($rule['destination']['not'])) {
				$line .= "to not $dst ";
			} else {
				$line .= "to $dst ";
			}
			
			if (in_array($rule['protocol'], array("tcp","udp"))) {
				
				if ($rule['destination']['port']) {
					$dstport = explode("-", $rule['destination']['port']);
					
					if ((!$dstport[1]) || ($dstport[0] == $dstport[1])) {
						$line .= "{$dstport[0]} ";
					} else {
						$line .= "{$dstport[0]}-{$dstport[1]} ";
					}
				}
			}
			
			if ($ispptp) {
				$line .= "on ng" . ($iif+1) . " ";
			} else {
				$line .= "via {$config['interfaces'][$rule['interface']]['if']}";
			}
			
			$line .= "\n";
			$shaperrules .= $line;
		}
			
		/* pipe configuration */
		$line = "pipe $pipei config bw {$rule['bandwidth']}Kbit/s ";
		
		if ($rule['delay']) {
			$line .= "delay {$rule['delay']} ";
		}
		
		switch ($rule['mask']) {
			case 'source':
				$line .= "mask src-ip 0xffffffff ";
				break;
			case 'destination':
				$line .= "mask dst-ip 0xffffffff ";
				break;
		}
		
		$line .= "\n";
		$shaperrules .= $line;
		
		$pipei++;
		$i++;
	}

	return $shaperrules;
}

?>
