#!/usr/local/bin/php
<?php 
/*
	services_dhcp.php
	part of m0n0wall (http://m0n0.ch/wall)
	
	Copyright (C) 2003 Manuel Kasper <mk@neon1.net>.
	All rights reserved.
	
	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions are met:
	
	1. Redistributions of source code must retain the above copyright notice,
	   this list of conditions and the following disclaimer.
	
	2. Redistributions in binary form must reproduce the above copyright
	   notice, this list of conditions and the following disclaimer in the
	   documentation and/or other materials provided with the distribution.
	
	THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
	INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
	AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
	AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
	OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
	SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
	INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
	CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
	ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
	POSSIBILITY OF SUCH DAMAGE.
*/

require("guiconfig.inc");

$pconfig['range_from'] = $config['dhcpd']['range']['from'];
$pconfig['range_to'] = $config['dhcpd']['range']['to'];
$pconfig['enable'] = isset($config['dhcpd']['enable']);

$lancfg = $config['interfaces']['lan'];

if (!is_array($config['dhcpd']['staticmap'])) {
	$config['dhcpd']['staticmap'] = array();
}
staticmaps_sort();
$a_maps = &$config['dhcpd']['staticmap'];

if ($_POST) {

	unset($input_errors);
	$pconfig = $_POST;

	/* input validation */
	if ($_POST['enable']) {
		$reqdfields = explode(" ", "range_from range_to");
		$reqdfieldsn = explode(",", "Range begin,Range end");
		
		do_input_validation($_POST, $reqdfields, $reqdfieldsn, &$input_errors);
		
		if (($_POST['range_from'] && !is_ipaddr($_POST['range_from']))) {
			$input_errors[] = "A valid range must be specified.";
		}
		if (($_POST['range_to'] && !is_ipaddr($_POST['range_to']))) {
			$input_errors[] = "A valid range must be specified.";
		}
		
		if (!$input_errors) {
			/* make sure the range lies within the current subnet */
			$subnet_start = (ip2long($lancfg['ipaddr']) & gen_subnet_mask_long($lancfg['subnet']));
			$subnet_end = (ip2long($lancfg['ipaddr']) | (~gen_subnet_mask_long($lancfg['subnet'])));
			
			if ((ip2long($_POST['range_from']) < $subnet_start) || (ip2long($_POST['range_from']) > $subnet_end) ||
			    (ip2long($_POST['range_to']) < $subnet_start) || (ip2long($_POST['range_to']) > $subnet_end)) {
				$input_errors[] = "The specified range lies outside of the current subnet.";	
			}
			
			if (ip2long($_POST['range_from']) > ip2long($_POST['range_to']))
				$input_errors[] = "The range is invalid (first element higher than second element).";
		}
	}

	if (!$input_errors) {
		$config['dhcpd']['range']['from'] = $_POST['range_from'];
		$config['dhcpd']['range']['to'] = $_POST['range_to'];
		$config['dhcpd']['enable'] = $_POST['enable'] ? true : false;
			
		write_config();
		
		$retval = 0;
		if (!file_exists($d_sysrebootreqd_path))
			$retval = services_dhcpd_configure();
		$savemsg = get_std_save_message($retval);
		
		if ($retval == 0) {
			if (file_exists($d_staticmapsdirty_path))
				unlink($d_staticmapsdirty_path);
		}
	}
}

if ($_GET['act'] == "del") {
	if ($a_maps[$_GET['id']]) {
		unset($a_maps[$_GET['id']]);
		write_config();
		touch($d_staticmapsdirty_path);
		header("Location: services_dhcp.php");
		exit;
	}
}
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<title>m0n0wall webGUI - Services: DHCP</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
<link href="gui.css" rel="stylesheet" type="text/css">
<script language="JavaScript">
<!--
function enable_change(enable_over) {
	if (document.iform.enable.checked || enable_over) {
		document.iform.range_from.disabled = 0;
		document.iform.range_to.disabled = 0;
	} else {
		document.iform.range_from.disabled = 1;
		document.iform.range_to.disabled = 1;
	}
}
//-->
</script>
</head>

<body link="#0000CC" vlink="#0000CC" alink="#0000CC">
<?php include("fbegin.inc"); ?>
<form action="services_dhcp.php" method="post" name="iform" id="iform">
<h3><font color="#777777">Services: DHCP</font></h3>
<?php if ($input_errors) print_input_errors($input_errors); ?>
<?php if ($savemsg) print_info_box(htmlspecialchars($savemsg)); ?>
<?php if (file_exists($d_staticmapsdirty_path)): ?><p>
<?php print_info_box_np("The static mapping configuration has been changed.<br>You must apply the changes in order for them to take effect.");?><br>
<input name="apply" type="submit" class="formbtn" id="apply" value="Apply changes"></p>
<?php endif; ?>			
              <table width="100%" border="0" cellpadding="6" cellspacing="0">
                <tr> 
                  <td valign="top" class="vtable">&nbsp;</td>
                  <td class="vtable"><input name="enable" type="checkbox" value="yes" <?php if ($pconfig['enable']) echo "checked"; ?> onClick="enable_change(false)">
                    Enable DHCP server on LAN interface</td>
				</tr>
                <tr> 
                  <td width="100" valign="top" class="vncellreq">Subnet</td>
                  <td class="vtable">
                    <?=gen_subnet($lancfg['ipaddr'], $lancfg['subnet']);?>
                  </td>
				</tr>
                <tr> 
                  <td width="100" valign="top" class="vncellreq">Subnet mask</td>
                  <td class="vtable">
                    <?=gen_subnet_mask($lancfg['subnet']);?>
                  </td>
                </tr>
                <tr> 
                  <td valign="top" class="vncellreq">Available range</td>
                  <td class="vtable">
				    <?php $lancfg = $config['interfaces']['lan']; ?>
                    <?=long2ip(ip2long($lancfg['ipaddr']) & gen_subnet_mask_long($lancfg['subnet']));?> - 
					<?=long2ip(ip2long($lancfg['ipaddr']) | (~gen_subnet_mask_long($lancfg['subnet']))); ?>
                  </td>
                </tr>
                <tr> 
                  <td width="100" valign="top" class="vncellreq">Range</td>
                  <td class="vtable"> <input name="range_from" type="text" class="formfld" id="range_from" size="20" value="<?=htmlspecialchars($pconfig['range_from']);?>"> 
                    &nbsp;to&nbsp; <input name="range_to" type="text" class="formfld" id="range_to" size="20" value="<?=htmlspecialchars($pconfig['range_to']);?>"></td>
                </tr>
                <tr> 
                  <td valign="top">&nbsp;</td>
                  <td> <input name="Submit" type="submit" class="formbtn" value="Save" onclick="enable_change(true)"> 
                  </td>
                </tr>
                <tr> 
                  <td valign="top">&nbsp;</td>
                  <td><p><span class="vexpl"><span class="red"><strong>Note:<br>
                      </strong></span>The DNS servers entered in <a href="system.php">System: 
                      General setup</a> (or the <a href="services_dnsmasq.php">DNS 
                      forwarder</a>, if enabled)<br>
                      </span><span class="vexpl">will be assigned to clients by 
                      the DHCP server.<br>
                      <br>
                      You may enter static mappings between IP and MAC addresses 
                      below. </span></p>
                    </td>
                </tr>
              </table>
			  &nbsp;<br>
              <table width="100%" border="0" cellpadding="0" cellspacing="0">
                <tr>
                  <td width="20%" class="listhdrr">IP address</td>
                  <td width="35%" class="listhdrr">MAC address </td>
                  <td width="35%" class="listhdrr">Description</td>
                  <td width="10%" class="list"></td>
				</tr>
			  <?php $i = 0; foreach ($a_maps as $mapent): ?>
                <tr>
                  <td class="listlr">
                    <?=htmlspecialchars($mapent['ipaddr']);?>
                  </td>
                  <td class="listr">
                    <?=htmlspecialchars($mapent['mac']);?>
                  </td>
                  <td class="listbg">
                    <?=htmlspecialchars($mapent['descr']);?>&nbsp;
                  </td>
                  <td valign="middle" nowrap class="list"> <a href="services_dhcp_edit.php?id=<?=$i;?>"><img src="e.gif" width="17" height="17" border="0"></a>
                     &nbsp;<a href="services_dhcp.php?act=del&id=<?=$i;?>" onclick="return confirm('Do you really want to delete this mapping?')"><img src="x.gif" width="17" height="17" border="0"></a></td>
				</tr>
			  <?php $i++; endforeach; ?>
                <tr> 
                  <td class="list" colspan="3"></td>
                  <td class="list"> <a href="services_dhcp_edit.php"><img src="plus.gif" width="17" height="17" border="0"></a></td>
				</tr>
              </table>
</form>
<script language="JavaScript">
<!--
enable_change(false);
//-->
</script>
<?php include("fend.inc"); ?>
</body>
</html>
